create_venv_info_sql = '''
CREATE TABLE IF NOT EXISTS venv_info(
    id INTEGER PRIMARY KEY ASC AUTOINCREMENT,
    project_path VARCHAR(300) NOT NULL,
    venv_name VARCHAR(200) UNIQUE,
    created_date DATE NOT NULL,
    requirement_file VARCHAR(300),
    connect_status VARCHAR(20) NOT NULL
);
'''

insert_into_venv_info_sql = '''
INSERT INTO venv_info(project_path, venv_name, created_date, requirement_file, connect_status) 
VALUES (?, ?, ?, ?, ?);
'''

update_connect_status_venv_info_sql = '''
UPDATE venv_info
SET connect_status = ?
WHERE venv_name = ?;
'''



from .utils.ui_controler import *
from .utils.all_system_control import check_system
import warnings
from .utils.package_directory_manager import *

__version__ = '0.0.1'

def main():
    warnings.filterwarnings("ignore")
    create_database_dir()

    THIS_MACHINE = check_system()
    end_program = False

    while not end_program:
        introduction_screen()
        end_program = menu_screen()








# from utils.all_system_control import *
# from utils.ANSI_color import *
# from utils.system_control_protocol import *
# from utils.string_processing import *

from .all_system_control import *
from .ANSI_color import *
from .system_control_protocol import *
from .string_processing import *





if __name__ != '__main__':
    print('\n\n\n\n\n\n\n\n\n')
    print('Checking UI system...')
    check_ui_library()
    import inquirer
    from inquirer.themes import GreenPassion

    print('Checking Operating System...')
    if check_system() == 'macos':
        # from utils.macos_system_control import *
        from .macos_system_control import *
    elif check_system() == 'win':
        # from utils.window_system_control import *
        from .window_system_control import *
    # from utils.ANSI_line_modification import *
    from .ANSI_line_modification import *
    

def test_name():
    print(__name__)


def introduction_screen():
    clear_screen()
    print('_______________________________________')
    print(f'        {get_color_str('Python VENV Generator', 'GREEN')}     ')
    print()
    print(f' This Version:\t{get_this_project_version()}')
    print(' Created By:\tNguyen Tuan')
    print(f' Your OS:\t{check_system()}')
    print('_______________________________________')


def create_new_project_screen(first_time = True, step = 'venv', *args, **kwargs):
    introduction_screen()
    print(f'            {get_color_str('Create New VENV', 'GREEN')}            ')
    print()




    if step == 'project_directory':
        if not first_time:
            print(f'{get_color_str('Please fill in the valid Directory Name', 'RED')}')
        
        sha_256_str = sha256_generator()
        print(f'sha256: {sha_256_str}')
        print('Paste this to answer to go back to main menu')
        print()
        print()
        print()
        project_directory = input(f'Libraries / Dependencies in requirements.txt (Y/n):{GO_UP_ONE_LINE}Project Name / Venv Name:{GO_UP_ONE_LINE}Project Directory: ')
        print()
        print()
        print()

        if project_directory == sha_256_str:
            return GO_BACK_TO_MENU_PAGE

        else:
            directory_exist = os.system(f'cd "{project_directory}"')
            if directory_exist == 0:
                return project_directory
            else:
                return None


    elif step == 'venv':


        if not first_time:
            print(f'{get_color_str('Please fill in the Venv Name', 'RED')}')
        
        sha_256_str = sha256_generator()
        print(f'sha256: {sha_256_str}')
        print('Paste this to answer to go back to main menu')
        print()
        print('Project Directory: ' + kwargs['project_directory'])
        print()
        

        project_name = input(f'Libraries / Dependencies in requirements.txt (Y/n):{GO_UP_ONE_LINE}Project Name / Venv Name: ')
        print()
        print()

        if project_name == sha_256_str:
            return GO_BACK_TO_MENU_PAGE
        else:
            return project_name
        

    elif step == 'requirement':
        if not first_time:
            print(f'{get_color_str('Please fill in the Valid Answer', 'RED')}')
        
        sha_256_str = sha256_generator()
        print(f'sha256: {sha_256_str}')
        print('Paste this to your answer to go back to main menu')
        print()
        print('Project Directory: ' + kwargs['project_directory'])
        print('Project Name / Venv Name: ' + kwargs['venv'])
        yes_no_requirement = input(f'Libraries / Dependencies in requirements.txt (Y/n): ')

        if yes_no_requirement == sha_256_str:
            return GO_BACK_TO_MENU_PAGE
        
        elif yes_no_requirement == 'Y' or yes_no_requirement == 'y':
            requirement_path = input('requirements.txt path: ')
            
            if requirement_path == None or requirement_path == '':
                return NO_REQUIREMENTS_FILE
            elif requirement_path == sha_256_str:
                return GO_BACK_TO_MENU_PAGE
            else:
                try:
                    with open(requirement_path, 'r') as f:
                        f.readline()
                except FileNotFoundError as e:
                    return None

            return requirement_path
        
        elif yes_no_requirement == 'N' or yes_no_requirement == 'n':
            return NO_REQUIREMENTS_FILE
        
        else:
            return None
        
    elif step == 'library_manual_input':

        if not first_time:
            print(f'{get_color_str('Please fill in the Valid Answer', 'RED')}')
        
        sha_256_str = sha256_generator()
        print(f'sha256: {sha_256_str}')
        print('Paste this to your answer to go back to main menu')
        print()
        print('Project Directory: ' + kwargs['project_directory'])
        print('Project Name / Venv Name: ' + kwargs['venv'])
        print(f'Libraries / Dependencies in requirements.txt (Y/n): n')
        yes_no_libraries = input('Set up venv with libraries (Y/n): ')

        if yes_no_libraries == sha_256_str:
            return GO_BACK_TO_MENU_PAGE
        
        elif yes_no_libraries == 'Y' or yes_no_libraries == 'y':
            libraries = input('Libraries: ')

            if libraries == None or libraries == '':
                return NO_LIBRARIES
            elif libraries == sha_256_str:
                return GO_BACK_TO_MENU_PAGE

            libraries = str_to_list(libraries)
            libraries = list_to_str(libraries, sep = ' ')
        
        elif yes_no_libraries == 'N' or yes_no_libraries == 'n':
            return NO_LIBRARIES

        else:
            return None


        return libraries
    

def view_venv_info_screen(first_time = True, step = None, *arg, **kwargs):
    introduction_screen()
    print(f'         {get_color_str('View VENV Details', 'GREEN')}            ')
    print()

    if step == 'view_venv_menu':
        view_choice = [inquirer.List('view_choice', message = 'Options', choices = [
            '1. View All Venv',
            '2. View 10 Recently Created Venv',
            '3. Edit Venv',
            '4. Back'
        ], default = '1. View All Venv')]

        user_answer = inquirer.prompt(view_choice, theme = GreenPassion())['view_choice']

        if user_answer[0] == '4':
            clear_screen()
            return False
        
        return user_answer[0]






def menu_screen():
    print(f'              {get_color_str('MENU', 'GREEN')}                     ')
    print()
    menu_choice = [inquirer.List('menu_choice', message = 'Options', choices = [
        '1. Create New Project',
        '2. View Created Virtual Enviroment',
        '3. Settings',
        '4. Exit'
    ], default = '1. Create New Project')]
    user_answer = inquirer.prompt(menu_choice, theme = GreenPassion())['menu_choice']

    display_functions = [create_new_project_screen, view_venv_info_screen]

    end_program = menu_choice_process(user_answer[0], display_functions)
    # print(user_answer[0])





    if user_answer[0] == '4':
        clear_screen()
        return True
    
    return end_program


import sqlite3
import pandas as pd
# from utils.sql_command import *

from .sql_command import *
'''
Database:

venv_info(id, project_path, venv_name, created_date, requirement_file, connect_status)

'''

def check_database(con):
    cur = con.cursor()
    cur.execute(create_venv_info_sql)

def connect_check_database(database_path: str):
    con = sqlite3.connect(database_path)
    check_database(con)
    return con


def insert_data(con: sqlite3.Connection, *args, **kwargs):
    cur = con.cursor()
    cur.execute(insert_into_venv_info_sql, (kwargs['project_path'], kwargs['venv_name'], kwargs['created_date'], kwargs['requirement_file'], kwargs['connect_status']))
    con.commit()

def update_data(con: sqlite3.Connection, *args, **kwargs):
    cur = con.cursor()
    cur.execute(update_connect_status_venv_info_sql, (kwargs['connect_status'], kwargs['venv_name']))



# from utils.system_control_protocol import *
# from utils.database_manager import *

from .system_control_protocol import *
from .database_manager import *
from datetime import date
from .package_directory_manager import *
import sqlite3 


def option1(display_function, *args, **kwargs):
    print(get_database_dir())
    con = connect_check_database(get_database_path())

    first_time = True
    project_directory = None

    while first_time or (not first_time and (project_directory == None or project_directory == '')):
        project_directory = display_function(first_time, 'project_directory')
        first_time = False
    
        if project_directory == GO_BACK_TO_MENU_PAGE:
            return False
    
    first_time = True
    venv_name = None
    
    while first_time or (not first_time and (venv_name == None or venv_name == '')):
        venv_name = display_function(first_time, 'venv', project_directory = project_directory)
        first_time = False

        if venv_name == GO_BACK_TO_MENU_PAGE:
            return False
        
    first_time = True
    require_name = None

    while first_time or (not first_time and (require_name == None or require_name == '')):
        require_name = display_function(first_time, 'requirement', venv = venv_name, project_directory = project_directory)
        first_time = False

        if require_name == GO_BACK_TO_MENU_PAGE:
            return False
        
        if require_name != NO_REQUIREMENTS_FILE:
            try:
                with open(require_name, 'r') as f:
                    f.readline()
            except FileNotFoundError as e:
                require_name = None
    
    if require_name == NO_REQUIREMENTS_FILE:
        first_time = True
        libraries = None

        while first_time or (not first_time and (libraries == None or libraries == '')):
            libraries = display_function(first_time, 'library_manual_input', venv = venv_name, project_directory = project_directory)
            first_time = False

            if libraries == GO_BACK_TO_MENU_PAGE:
                return False

        kwargs['system_control'].create_venv(project_directory, venv_name)
        insert_data(con, project_path = project_directory, venv_name = venv_name, created_date = date.today(), requirement_file = None, connect_status = 'yes')

        if libraries != NO_LIBRARIES:
            kwargs['system_control'].install_libraries(project_directory, venv_name, libraries)
        return False
    
    if require_name != NO_REQUIREMENTS_FILE:
        kwargs['system_control'].create_venv(project_directory, venv_name)
        kwargs['system_control'].install_libraries_with_requirement(project_directory, venv_name, require_name)
        insert_data(con, project_path = project_directory, venv_name = venv_name, created_date = date.today(), requirement_file = require_name, connect_status = 'yes')
        


    else:
        kwargs['system_control'].create_venv(project_directory, venv_name)
        insert_data(con, project_path = project_directory, venv_name = venv_name, created_date = date.today(), requirement_file = None, connect_status = 'yes')

    con.close()
    
def option2(display_function, *args, **kwargs):
    con = connect_check_database(get_database_path())







    pass


print('Importing system control...', end = '                                          \r')
from . import all_system_control

print('Importing ANSI color...', end = '                                              \r')
from . import ANSI_color

print('Importing ANSI line modification...', end = '                                  \r')
from . import ANSI_line_modification

print('Importing database manager...', end = '                                        \r')
from . import database_manager

print('Importing macos system control...', end = '                                    \r')
from . import macos_system_control

print('Importing option manager...', end = '                                          \r')
from . import option_manager

print('Importing sql command...', end = '                                             \r')
from . import sql_command

print('Importing string_processing', end = '                                          \r')
from . import string_processing

print('Importing system control protocol...', end = '                                 \r')
from . import system_control_protocol

print('Importing UI controler...', end = '                                            \r')
from . import ui_controler

print('Importing Window system control', end = '                                      \r')
from . import window_system_control

print('Importing package directory manager...', end = '                                   \r')
from . import package_directory_manager


# from utils.ANSI_color import *
# from utils.system_control_protocol import *
# from utils.option_manager import *
# from utils import macos_system_control as MacOS
# from utils import window_system_control as Windows


from .ANSI_color import *
from .system_control_protocol import *
from .option_manager import *
from . import macos_system_control as MacOS
from . import window_system_control as Windows
import platform
import os
import random
import hashlib



__version__ = '1.0'



def check_system():
    if platform.system() == 'Darwin':
        return 'macos'
    elif platform.system() == 'win32' or platform.system() == 'Windows':
        return 'win'
    
def check_ui_library():
    try:
        import inquirer
    except ModuleNotFoundError as e:
        print()
        print(f'{get_color_str('inquirer', 'RED')} module have not yet installed')
        print()
        print('Do you want to download it to your machine?')
        print()
        print('The library is Required for the program to run')
        userChoice = input('Your choice (Y/n): ')
        if userChoice == 'Y' or userChoice == 'y':
            os.system('pip install inquirer')
            print()
            print('Please rerun the program after installation')
            exit()        
        else:
            print()
            print_color('Cannot process further due to the lack of library', 'RED')
            exit()

def sha256_generator():
    return hashlib.sha256(str(random.randint(-100000000000, 10000000000000)).encode('utf-8')).hexdigest()


    

def menu_choice_process(user_answer, display_function):
    if check_system() == 'macos':
        os_system_control = MacOS
    elif check_system() == 'win':
        os_system_control = Windows


    if user_answer == '1':
        return option1(display_function[0], system_control = os_system_control)
    elif user_answer == '2':
        return option2(display_function[1], system_control = os_system_control)






def get_this_project_version():
    return __version__